<?php
/**
 * @file
 *  Drupal State module
 */

/**
 * Implements drupal_get_form().
 *
 * Defines configuration form fields.
 *
 * @param array $form
 * @param array &$form_state
 * @return array
 *   - the return value of system_settings_form()
 */
function _state_admin_form($form, &$form_state) {
  drupal_add_css(
      ($path = drupal_get_path('module', 'state')) . '/admin/state.admin.css',
      array('type' => 'file', 'group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );
  drupal_add_js(
      $path . '/admin/state.admin.js',
      array('type' => 'file', 'group' => JS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );

  $timeout_cookie = (int) ini_get('session.cookie_lifetime');
  $timeout_server = (int) ini_get('session.gc_maxlifetime');
  $timeout_warning = '';
  if ($timeout_cookie) {
    $timeout_warning .= '<p class="warning">' . t('Session prolongation isn\'t applicable when the session cookie doesn\'t expire when the browser is closed, check session.cookie_lifetime in settings.php (should be zero).') . '</p>';
  }
  if ($timeout_server > state::FORM_EXPIRE) {
    $timeout_warning .= '<p class="warning">'
        . t(
            'Current session timeout (life on server) is !timeout minutes. A lower value, like 1800 (a half an hour) is recommended.!breakA Drupal form expires after !hrs hours, thus a higher value than that makes little sense. Check session.gc_maxlifetime in settings.php.',
            array('!break' => '<br/>', '!timeout' => round($timeout_server / 60, 1), '!hrs' => state::FORM_EXPIRE / (60 * 60))
        ) . '</p>';
  }

  $form['help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Help'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'content' => array(
        '#type' => 'markup',
        '#markup' =>
'<h5></h5>
<p></p>
<h5>' . t('Getting close to session/form expiration') . '</h5>
<p>' . t('10 to 20% before session/form expiration frontend State pops a dialog box telling the user that the page is about to expire - \'please save your work...\'.'
) . '</p>
<h5>' . t('Form has expired') . '</h5>
<p>' . t('If a page\'s form has expired, State will always attempt to !emphreload!_emph the page.
!breakIf browser settings prevent the reload, State will instead pop a dialog box.',
    array('!break' => '<br/>', '!emph' => '<em>', '!_emph' => '</em>')
) . '</p>
<h5>' . t('Session has expired, or isn\'t current - redirect to home or reload current page?') . '</h5>
<p>' . t('If a window/tab\'s session has expired - or the visitor has a new session or no session at all:
!break- State will attempt to either !emphredirect to home!_emph (default) or !emphreload!_emph the page.
!paragraphRedirecting to home is preferable when a page may require user login, and it isn\'t certain that such a page displays a login form when the visitor isn\'t logged in.
!breakHowever, redirecting to home !emphdoes!_emph break the visitor\'s navigational flow.
!paragraphReloading the page is far more user-friendly, unless the visitor simply meets a 403 (not allowed to use this page).',
    array('!break' => '<br/>', '!paragraph' => '</p><p>', '!emph' => '<em>', '!_emph' => '</em>')
) . '</p>
<h5>' . t('Perpetual redirects/reloads?') . '</h5>
<p>' . t(
    'Possible for form expiration. Not possible for session expiration, because State destroys a session when it expires.',
    array('!break' => '<br/>', '!paragraph' => '</p><p>')
) . '</p>
<h5>' . t('Check interval') . '</h5>
<p>' . t(
    'For every N seconds, check that window/tab session is alive and current, and form hasn\'t expired,
!breakand - if user-activity based session prolongation - whether the visitor has been active since last check.
!breakRecommended value, if session timeout (life on server) is 1800 (half an hour) or more: 300 (~ 5 minutes).
!breakState will also perform AJAX requests at this interval, whenever the prolongation conditions are met.
!paragraphSetting this value correctly is a matter of balancing contradictory interests. Too low: too many requests to the server.
!breakToo high: user-activity based prolongation will tend to prolong sessions excessively (because the last activity may have taken place almost the interval time ago).
!paragraphFrontend ensures that the interval is maximally 20% of the session timeout.
!breakAnd the actual intervals will fluctuate approx. +/-20%, due to fuzzy logic algorithm which ensures that the browser also only "calls in" at the set interval when multiple tabs are open.',
    array('!break' => '<br/>', '!paragraph' => '</p><p>')
) . '</p>'
      ),
  );
  $form['frontend_session_management'] = array(
      '#type' => 'fieldset',
      '#title' => t('Frontend session management'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      'module_state_frntnd' => array(
          '#type' => 'checkbox',
          '#title' => t('Use frontend session management'),
          '#default_value' => variable_get('module_state_frntnd', 0),
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
          '#prefix' => '<p>' . t('Session cookie life:') . ' ' . (!$timeout_cookie ? t('session') : ($timeout_cookie . ' ' . t('seconds')))
              . '<br/>'
              . t('Session timeout (life on server):') . ' ' . $timeout_server . ' ' . t('seconds')
              . '</p>'
              . $timeout_warning,
      ),
      'frontend_session_configure' => array(
          '#type' => 'fieldset',
          '#title' => t('Configure frontend session management'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          'module_state_enovrrd' => array(
              '#type' => 'textarea',
              '#default_value' => variable_get('module_state_enovrrd', ''),
              '#maxlength' => 10000,
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
          ),
          'sub_fields__module_state_frntndsttngs' => array(
              '#type' => 'markup',
              '#markup' =>
'<div class="form-item form-type-textarea frntndsttngs">
<div class="form-textarea-wrapper textarea-processed"><textarea name="frntndsttngs" id="frntndsttngs" autocomplete="off">'
              . json_encode(variable_get('module_state_frntndsttngs', new stdClass()))
              . '</textarea></div>
</div>
<div class="form-item form-type-textarea prlngactrls">
<label>' . t('Prolong session based on recent visitor activity (like mouse move), for visitors having any of these roles') . '</label>
<div class="form-textarea-wrapper resizable textarea-processed resizable-textarea"><textarea class="form-textarea" rows="1" cols="100" name="prlngactrls" id="edit-prlngactrls" autocomplete="off"' . (!$timeout_cookie ? '' : ' disabled="disabled"') . '>administrator</textarea><div class="grippie"></div></div>
<div class="description">' . t('* means all roles, even anonymous user. If a visitor has any of these roles, unconditional session prolongiation (next option) is ignored - except if this is *. Comma-separated.') . '</div>
</div>
<div class="form-item form-type-textarea prlngrls">
  <label>' . t('Prolong session unconditionally, for visitors having any of these roles') . '</label>
 <div class="form-textarea-wrapper resizable textarea-processed resizable-textarea"><textarea class="form-textarea" rows="1" cols="100" name="prlngrls" id="edit-prlngrls" autocomplete="off"' . (!$timeout_cookie ? '' : ' disabled="disabled"') . '>*</textarea><div class="grippie"></div></div>
<div class="description">' . t('!emphThis mode is not recommended for administrative roles.!_emph Doesn\'t apply to user profile editing pages; degrades to visitor activity based prolongiation. * means all roles, even anonymous user. Comma-separated.', array('!emph' => '<em>', '!_emph' => '</em>')) . '</div>
</div>
<div class="form-item form-type-checkbox chcksss">
<input type="checkbox" class="form-checkbox" value="1" name="chcksss" id="edit-chcksss" autocomplete="off"><label class="option">' . t('For visitors that don\'t get frontend session prolongation - check that session is current and form hasn\'t expired') . '</label>
<div class="description">' . t('Applies to any visitor that has a session, but isn\'t logged-in or hasn\'t got a role matched by the session prolongation role patterns.') . '</div>
</div>
<div class="form-item form-type-checkbox sssrld">
<input type="checkbox" class="form-checkbox" value="1" name="sssrld" id="edit-sssrld" autocomplete="off"><label class="option">' . t('Reload current page at session expiration, instead of redirecting to start page') . '</label>
</div>
<div class="form-item form-type-textfield chckntrvl">
<label>' . t('Check interval - see help') . '</label>
<input type="text" class="form-text" maxlength="128" size="4" value="300" name="chckntrvl" id="edit-chckntrvl" style="text-align: right;" autocomplete="off">
<div class="description">' . t('Default: !secs seconds (~ !mins minutes)', array('!secs' => 300, '!mins' => 5)) . '</div>
</div>'
              .
              //  Override English
              '<div class="title-local-en-override">' . t('Override English session/form expire warnings') . '</div>'
              . '<div class="description-local-en-override">' . t('When translating to other languages, look for the defaults, not these overrides (if any).') . '</div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form about to expire') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page is about to expire!newline!newlinePlease save now,!newlineif you have made any changes.!newlineOtherwise, please !link_startreload the page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__preExpire" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline'))
              . '</div></div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form expired - redirect to start page') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page has expired!newline!newlinePlease !link_startproceed to the start page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__expired_redirect" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline, !url'))
              . '<br/>'
              . t('NB: The visitor will only see this message if the browser prohibits Javascript redirect(Internet Explorer: \'META refresh\').')
              . '</div></div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form expired - reload current page') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page has expired!newline!newlinePlease !link_startreload the page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__expired_reload" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline, !url'))
              . '<br/>'
              . t('NB: The visitor will only see this message if the browser prohibits Javascript redirect (Internet Explorer: \'META refresh\').')
              . '</div></div>',
          )
      )
  );
  $form['request_counting'] = array(
      '#type' => 'fieldset',
      '#title' => t('Detect request type - page versus AJAX'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      'module_state_cntreqs' => array(
          '#type' => 'checkbox',
          '#title' => t('Detect request type and count requests'),
          '#default_value' => variable_get('module_state_cntreqs', 0),
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
      ),
      'module_state_nonpages' => array(
          '#type' => 'textarea',
          '#title' => t('List of non-page paths - like iframed page views, and AJAX calls failing to send the HTTP_X_REQUESTED_WITH header'),
          '#description' => t(
              'Newline separated. Built-in non-page paths are:!break &nbsp; !built_ins',
              array('!break' => '<br/>', '!built_ins' => join('<br/> &nbsp; ', State::nonPagePaths(TRUE)))
           ),
          '#default_value' => variable_get('module_state_nonpages', ''),
          '#maxlength' => 1000,
          '#cols' => 100,
          '#rows' => 2,
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
      ),
  );
  $form['#submit'][] = 'state_admin_form_submit';
  return system_settings_form($form);
}

/**
 * @param array $form
 * @param array &$form_state
 * @return void
 */
function _state_admin_form_submit($form, &$form_state) {
  //  Get frontend settings from proxy JSON field, and pass to real settings object.
  variable_set(
      'module_state_frntndsttngs',
      isset($_POST['frntndsttngs']) && strlen($v = trim($_POST['frntndsttngs'])) ? json_decode($v) : new stdClass()
  );
  //  Fix formatting of non-page paths.
  if (strlen($v = trim($form_state['values']['module_state_nonpages']))) {
    $built_ins = State::nonPagePaths(TRUE);
    $paths = array();
    $le = count($a = explode("\n", str_replace("\r", '', $v)));
    for ($i = 0; $i < $le; $i++) {
      if (!strlen($v = trim($a[$i]))) {
        continue;
      }
      if ($v{0} == '/') { // Remove starting slash.
        $v = substr($v, 1);
      }
      if (!strlen($v)) {
        continue;
      }
      if ($v{ strlen($v) - 1 } != '/') { // Secure trailing slash.
        $v .= '/';
      }
      if ($v != '/' && !in_array($v, $built_ins)) {
        $paths[] = $v;
      }
    }
    $form_state['values']['module_state_nonpages'] = $paths ? join("\n", $paths) : '';
  }
}