/**
 * @file
 *	Drupal State module
 */

/**
 * @constructor
 * @class
 * @singleton
 * @param {obj} $
 *	- jQuery
 */
var State_Admin = function($) {
	var self = this,
  _frontendSettings = {},
  _localEnOverrides = {},
  _passValue_frontendSettings,
  _passValue_enOverride;

  /**
   * Pass value of a form field to object list of all (relevant) form fields, and update JSON object in aggregate form field.
   *
   * Updates aggregate value to empty string, if all fields are empty.
   *
   * @return {void}
   */
  _passValue_frontendSettings = function() {
    var k, nm = this.getAttribute("name"), v;
    switch(nm) {
      case 'prlngactrls':
      case 'prlngrls':
        _frontendSettings[nm] = (v = $.trim(this.value).replace(/\s*,\s*/g, ",")).indexOf("*") > -1 ? ["*"] : v.split(",");
        break;
      case 'chcksss':
        _frontendSettings[nm] = this.checked ? true : false;
        break;
      case 'chckntrvl':
        _frontendSettings[nm] = (v = $.trim(this.value)) && /^\d+$/.test(v) ? v : 300;
        break;
      case 'sssrld':
        _frontendSettings[nm] = this.checked ? true : false;
        break;
    }
    $("textarea[name='frntndsttngs']").get(0).value = JSON.stringify(_frontendSettings);
  };
  /**
   * Pass value of a form field to object list of all (relevant) form fields, and update JSON object in aggregate form field.
   *
   * Updates aggregate value to empty string, if all fields are empty.
   *
   * @return {void}
   */
  _passValue_enOverride = function() {
    var k, nonEmpty;
    _localEnOverrides[ this.getAttribute("name").replace(/local_en__/, "") ] = $.trim(this.value);
    for(k in _localEnOverrides) {
      if(_localEnOverrides.hasOwnProperty(k) && _localEnOverrides[k].length) {
        nonEmpty = true;
        break;
      }
    }
    $("textarea[name='module_state_enovrrd']").get(0).value = nonEmpty ? JSON.stringify(_localEnOverrides) : "";
  };
  /**
   * Unpack aggregate values, and set event handlers on all relevant form fields.
   *
   * @return {void}
   */
  this.init = function() {
    var jq, v, k, hasEnOverride;

    //  This module cannot be configured without JSON support (IE<8).
    if(!window.JSON || typeof JSON.stringify !== "function" || typeof JSON.parse !== "function") {
      alert(
          "This module cannot be configured using current browser\nbecause the browser has no JSON support." +
          ($.browser.msie ? "\n\nInternet Explorer <8 never supports JSON, version 8 may support it, version >8 always supports it." : "")
      );
      return;
    }

    //  Make frontend session management checkbox control expand/collapse of child fieldset.
    if((jq = $("input#edit-module-state-frntnd")).get(0).checked) {
      $("fieldset#edit-frontend-session-configure").removeClass("collapsed");
    }
    jq.click(function(){
      $("fieldset#edit-frontend-session-configure")[ (this.checked ? "remove" : "add") + "Class" ]("collapsed");
      //  When clicking on/off, trigger change on role fields, otherwise defaults will never get registered.
      $("textarea[name='prlngactrls']").trigger("change");
      $("textarea[name='prlngrls']").trigger("change");
    });

    //  Combine frontend settings fields into a single hidden textarea containing JSON object.
    if((v = $("textarea[name='frntndsttngs']").get(0).value)) {
      _frontendSettings = JSON.parse(v);
      for(k in _frontendSettings) {
        if(_frontendSettings.hasOwnProperty(k)) {
          switch(k) {
            case 'prlngactrls':
            case 'prlngrls':
              $("textarea[name='" + k + "']").get(0).value = _frontendSettings[k].join(",");
              break;
            case 'chcksss':
              if(_frontendSettings[k]) {
                $("input[name='" + k + "']").get(0).checked = "checked";
              }
              break;
            case 'chckntrvl':
              $("input[name='" + k + "']").get(0).value = _frontendSettings[k];
              break;
            case 'sssrld':
              if(_frontendSettings[k]) {
                $("input[name='" + k + "']").get(0).checked = "checked";
              }
              break;
          }
        }
      }
    }
    $("textarea[name='prlngactrls']").change(_passValue_frontendSettings);
    $("textarea[name='prlngrls']").change(_passValue_frontendSettings);
    $("input[name='chcksss']").change(_passValue_frontendSettings);
    $("input[name='chckntrvl']").change(_passValue_frontendSettings);
    $("input[name='sssrld']").change(_passValue_frontendSettings);

    //  Combine English message override fields into a single hidden textarea containing JSON object.
    if((v = $("textarea[name='module_state_enovrrd']").get(0).value)) {
      _localEnOverrides = JSON.parse(v);
      for(k in _localEnOverrides) {
        if(_localEnOverrides.hasOwnProperty(k)) {
          hasEnOverride = true;
          $("input[name='local_en__" + k + "']").get(0).value = _localEnOverrides[k];
        }
      }
    }
    if(hasEnOverride) {
      $("fieldset#edit-override-english").removeClass("collapsed");
    }
    $("input.local-en-override").each(function() {
      $(this).change(_passValue_enOverride);
    });
  };

};
//	instantiate and run init at window onload
(function($) {
	if(!$) {
		return;
	}
	$(document).bind("ready", function() {
		(State_Admin = new State_Admin($)).init();
	} );
})(jQuery);
